@@ -4,13 +4,14 @@
  * SPDX-License-Identifier: BSD-3-Clause
  */
 
-#ifndef _HARDWARE_DMA_H_
-#define _HARDWARE_DMA_H_
+#ifndef _HARDWARE_DMA_H
+#define _HARDWARE_DMA_H
 
 #include "pico.h"
 #include "hardware/structs/dma.h"
 #include "hardware/regs/dreq.h"
 #include "pico/assert.h"
+#include "hardware/regs/intctrl.h"
 
 #ifdef __cplusplus
 extern "C" {
@@ -19,11 +20,11 @@ extern "C" {
 /** \file hardware/dma.h
  *  \defgroup hardware_dma hardware_dma
  *
- * DMA Controller API
+ * \brief DMA Controller API
  *
- * The RP2040 Direct Memory Access (DMA) master performs bulk data transfers on a processor’s
+ * The RP-series microcontroller Direct Memory Access (DMA) master performs bulk data transfers on a processor’s
  * behalf. This leaves processors free to attend to other tasks, or enter low-power sleep states. The
- * data throughput of the DMA is also significantly higher than one of RP2040’s processors.
+ * data throughput of the DMA is also significantly higher than one of RP-series microcontroller’s processors.
  *
  * The DMA can perform one read access and one write access, up to 32 bits in size, every clock cycle.
  * There are 12 independent channels, which each supervise a sequence of bus transfers, usually in
@@ -34,20 +35,29 @@ extern "C" {
  * * Memory to memory
  */
 
-// these are not defined in generated dreq.h
-#define DREQ_DMA_TIMER0 DMA_CH0_CTRL_TRIG_TREQ_SEL_VALUE_TIMER0
-#define DREQ_DMA_TIMER1 DMA_CH0_CTRL_TRIG_TREQ_SEL_VALUE_TIMER1
-#define DREQ_DMA_TIMER2 DMA_CH0_CTRL_TRIG_TREQ_SEL_VALUE_TIMER2
-#define DREQ_DMA_TIMER3 DMA_CH0_CTRL_TRIG_TREQ_SEL_VALUE_TIMER3
-#define DREQ_FORCE      DMA_CH0_CTRL_TRIG_TREQ_SEL_VALUE_PERMANENT
+// PICO_CONFIG: PARAM_ASSERTIONS_ENABLED_HARDWARE_DMA, Enable/disable hardware_dma assertions, type=bool, default=0, group=hardware_dma
+#ifndef PARAM_ASSERTIONS_ENABLED_HARDWARE_DMA
+#ifdef PARAM_ASSERTIONS_ENABLED_DMA // backwards compatibility with SDK < 2.0.0
+#define PARAM_ASSERTIONS_ENABLED_HARDWARE_DMA PARAM_ASSERTIONS_ENABLED_DMA
+#else
+#define PARAM_ASSERTIONS_ENABLED_HARDWARE_DMA 0
+#endif
+#endif
 
-// PICO_CONFIG: PARAM_ASSERTIONS_ENABLED_DMA, Enable/disable DMA assertions, type=bool, default=0, group=hardware_dma
-#ifndef PARAM_ASSERTIONS_ENABLED_DMA
-#define PARAM_ASSERTIONS_ENABLED_DMA 0
+/**
+ * \def DMA_IRQ_NUM(n)
+ * \ingroup hardware_dma
+ * \hideinitializer
+ * \brief Returns the \ref irq_num_t for the nth DMA interrupt
+ *
+ * Note this macro is intended to resolve at compile time, and does no parameter checking
+ */
+#ifndef DMA_IRQ_NUM
+#define DMA_IRQ_NUM(irq_index) (DMA_IRQ_0 + (irq_index))
 #endif
 
 static inline void check_dma_channel_param(__unused uint channel) {
-#if PARAM_ASSERTIONS_ENABLED(DMA)
+#if PARAM_ASSERTIONS_ENABLED(HARDWARE_DMA)
     // this method is used a lot by inline functions so avoid code bloat by deferring to function
     extern void check_dma_channel_param_impl(uint channel);
     check_dma_channel_param_impl(channel);
@@ -55,7 +65,7 @@ static inline void check_dma_channel_param(__unused uint channel) {
 }
 
 static inline void check_dma_timer_param(__unused uint timer_num) {
-    valid_params_if(DMA, timer_num < NUM_DMA_TIMERS);
+    valid_params_if(HARDWARE_DMA, timer_num < NUM_DMA_TIMERS);
 }
 
 inline static dma_channel_hw_t *dma_channel_hw_addr(uint channel) {
@@ -157,7 +167,6 @@ static inline void channel_config_set_read_increment(dma_channel_config *c, bool
  * \param c Pointer to channel configuration object
  * \param incr True to enable write address increments, if false, each write will be to the same address
  *             Usually disabled for memory to peripheral transfers
- * Usually disabled for memory to peripheral transfers
  */
 static inline void channel_config_set_write_increment(dma_channel_config *c, bool incr) {
     c->ctrl = incr ? (c->ctrl | DMA_CH0_CTRL_TRIG_INCR_WRITE_BITS) : (c->ctrl & ~DMA_CH0_CTRL_TRIG_INCR_WRITE_BITS);
@@ -479,7 +488,7 @@ inline static void dma_channel_transfer_to_buffer_now(uint channel, volatile voi
  * \param chan_mask Bitmask of all the channels requiring starting. Channel 0 = bit 0, channel 1 = bit 1 etc.
  */
 static inline void dma_start_channel_mask(uint32_t chan_mask) {
-    valid_params_if(DMA, chan_mask && chan_mask < (1u << NUM_DMA_CHANNELS));
+    valid_params_if(HARDWARE_DMA, chan_mask && chan_mask < (1u << NUM_DMA_CHANNELS));
     dma_hw->multi_channel_trigger = chan_mask;
 }
 
@@ -497,10 +506,12 @@ static inline void dma_channel_start(uint channel) {
  *
  * Function will only return once the DMA has stopped.
  *
- * Note that due to errata RP2040-E13, aborting a channel which has transfers
+ * \if rp2040_specific
+ * RP2040 only: Note that due to errata RP2040-E13, aborting a channel which has transfers
  * in-flight (i.e. an individual read has taken place but the corresponding write has not), the ABORT
  * status bit will clear prematurely, and subsequently the in-flight
  * transfers will trigger a completion interrupt once they complete.
+ *\endif
  *
  * The effect of this is that you \em may see a spurious completion interrupt
  * on the channel as a result of calling this method.
@@ -523,6 +534,11 @@ static inline void dma_channel_start(uint channel) {
  *  dma_channel_set_irq0_enabled(channel, true);
  *\endcode
  *
+ * \if rp2350_specific
+ * RP2350 only: Due to errata RP12350-E5 (see the RP2350 datasheet for further detail), it is necessary to clear the enable bit of
+ * the aborted channel and any chained channels prior to the abort to prevent re-triggering.
+ * \endif
+ *
  * \param channel DMA channel
  */
 static inline void dma_channel_abort(uint channel) {
@@ -599,12 +615,12 @@ static inline void dma_set_irq1_channel_mask_enabled(uint32_t channel_mask, bool
  * \param enabled true to enable interrupt via irq_index for specified channel, false to disable.
  */
 static inline void dma_irqn_set_channel_enabled(uint irq_index, uint channel, bool enabled) {
-    invalid_params_if(DMA, irq_index > 1);
-    if (irq_index) {
-        dma_channel_set_irq1_enabled(channel, enabled);
-    } else {
-        dma_channel_set_irq0_enabled(channel, enabled);
-    }
+    invalid_params_if(HARDWARE_DMA, irq_index >= NUM_DMA_IRQS);
+
+    if (enabled)
+        hw_set_bits(&dma_hw->irq_ctrl[irq_index].inte, 1u << channel);
+    else
+        hw_clear_bits(&dma_hw->irq_ctrl[irq_index].inte, 1u << channel);
 }
 
 /*! \brief  Enable multiple DMA channels' interrupt via either DMA_IRQ_0 or DMA_IRQ_1
@@ -615,11 +631,11 @@ static inline void dma_irqn_set_channel_enabled(uint irq_index, uint channel, bo
  * \param enabled true to enable all the interrupts specified in the mask, false to disable all the interrupts specified in the mask.
  */
 static inline void dma_irqn_set_channel_mask_enabled(uint irq_index, uint32_t channel_mask,  bool enabled) {
-    invalid_params_if(DMA, irq_index > 1);
-    if (irq_index) {
-        dma_set_irq1_channel_mask_enabled(channel_mask, enabled);
+    invalid_params_if(HARDWARE_DMA, irq_index >= NUM_DMA_IRQS);
+    if (enabled) {
+        hw_set_bits(&dma_hw->irq_ctrl[irq_index].inte, channel_mask);
     } else {
-        dma_set_irq0_channel_mask_enabled(channel_mask, enabled);
+        hw_clear_bits(&dma_hw->irq_ctrl[irq_index].inte, channel_mask);
     }
 }
 
@@ -653,9 +669,9 @@ static inline bool dma_channel_get_irq1_status(uint channel) {
  * \return true if the channel is a cause of the DMA_IRQ_N, false otherwise
  */
 static inline bool dma_irqn_get_channel_status(uint irq_index, uint channel) {
-    invalid_params_if(DMA, irq_index > 1);
+    invalid_params_if(HARDWARE_DMA, irq_index >= NUM_DMA_IRQS);
     check_dma_channel_param(channel);
-    return (irq_index ? dma_hw->ints1 : dma_hw->ints0) & (1u << channel);
+    return dma_hw->irq_ctrl[irq_index].ints & (1u << channel);
 }
 
 /*! \brief  Acknowledge a channel IRQ, resetting it as the cause of DMA_IRQ_0
@@ -665,7 +681,7 @@ static inline bool dma_irqn_get_channel_status(uint irq_index, uint channel) {
  */
 static inline void dma_channel_acknowledge_irq0(uint channel) {
     check_dma_channel_param(channel);
-    hw_set_bits(&dma_hw->ints0, (1u << channel));
+    dma_hw->ints0 = 1u << channel;
 }
 
 /*! \brief  Acknowledge a channel IRQ, resetting it as the cause of DMA_IRQ_1
@@ -675,7 +691,7 @@ static inline void dma_channel_acknowledge_irq0(uint channel) {
  */
 static inline void dma_channel_acknowledge_irq1(uint channel) {
     check_dma_channel_param(channel);
-    hw_set_bits(&dma_hw->ints1, (1u << channel));
+    dma_hw->ints1 = 1u << channel;
 }
 
 /*! \brief  Acknowledge a channel IRQ, resetting it as the cause of DMA_IRQ_N
@@ -685,9 +701,9 @@ static inline void dma_channel_acknowledge_irq1(uint channel) {
  * \param channel DMA channel
  */
 static inline void dma_irqn_acknowledge_channel(uint irq_index, uint channel) {
-    invalid_params_if(DMA, irq_index > 1);
+    invalid_params_if(HARDWARE_DMA, irq_index >= NUM_DMA_IRQS);
     check_dma_channel_param(channel);
-    hw_set_bits(irq_index ? &dma_hw->ints1 : &dma_hw->ints0, (1u << channel));
+    dma_hw->irq_ctrl[irq_index].ints = 1u << channel;
 }
 
 /*! \brief  Check if DMA channel is busy
@@ -698,7 +714,7 @@ static inline void dma_irqn_acknowledge_channel(uint irq_index, uint channel) {
  */
 inline static bool dma_channel_is_busy(uint channel) {
     check_dma_channel_param(channel);
-    return !!(dma_hw->ch[channel].al1_ctrl & DMA_CH0_CTRL_TRIG_BUSY_BITS);
+    return dma_hw->ch[channel].al1_ctrl & DMA_CH0_CTRL_TRIG_BUSY_BITS;
 }
 
 /*! \brief  Wait for a DMA channel transfer to complete
@@ -708,7 +724,7 @@ inline static bool dma_channel_is_busy(uint channel) {
  */
 inline static void dma_channel_wait_for_finish_blocking(uint channel) {
     while (dma_channel_is_busy(channel)) tight_loop_contents();
-    // stop the compiler hoisting a non volatile buffer access above the DMA completion.
+    // stop the compiler hoisting a non-volatile buffer access above the DMA completion.
     __compiler_memory_barrier();
 }
 
@@ -738,9 +754,13 @@ inline static void dma_sniffer_enable(uint channel, uint mode, bool force_channe
     if (force_channel_enable) {
         hw_set_bits(&dma_hw->ch[channel].al1_ctrl, DMA_CH0_CTRL_TRIG_SNIFF_EN_BITS);
     }
-    dma_hw->sniff_ctrl = ((channel << DMA_SNIFF_CTRL_DMACH_LSB) & DMA_SNIFF_CTRL_DMACH_BITS) |
-                         ((mode << DMA_SNIFF_CTRL_CALC_LSB) & DMA_SNIFF_CTRL_CALC_BITS) |
-                         DMA_SNIFF_CTRL_EN_BITS;
+    hw_write_masked(&dma_hw->sniff_ctrl,
+        (((channel << DMA_SNIFF_CTRL_DMACH_LSB) & DMA_SNIFF_CTRL_DMACH_BITS) |
+         ((mode << DMA_SNIFF_CTRL_CALC_LSB) & DMA_SNIFF_CTRL_CALC_BITS) |
+         DMA_SNIFF_CTRL_EN_BITS),
+        (DMA_SNIFF_CTRL_DMACH_BITS |
+         DMA_SNIFF_CTRL_CALC_BITS |
+         DMA_SNIFF_CTRL_EN_BITS));
 }
 
 /*! \brief Enable the Sniffer byte swap function
@@ -761,6 +781,36 @@ inline static void dma_sniffer_set_byte_swap_enabled(bool swap) {
         hw_clear_bits(&dma_hw->sniff_ctrl, DMA_SNIFF_CTRL_BSWAP_BITS);
 }
 
+/*! \brief Enable the Sniffer output invert function
+ *  \ingroup hardware_dma
+ *
+ * If enabled, the sniff data result appears bit-inverted when read.
+ * This does not affect the way the checksum is calculated.
+ *
+ * \param invert Set true to enable output bit inversion
+ */
+inline static void dma_sniffer_set_output_invert_enabled(bool invert) {
+    if (invert)
+        hw_set_bits(&dma_hw->sniff_ctrl, DMA_SNIFF_CTRL_OUT_INV_BITS);
+    else
+        hw_clear_bits(&dma_hw->sniff_ctrl, DMA_SNIFF_CTRL_OUT_INV_BITS);
+}
+
+/*! \brief Enable the Sniffer output bit reversal function
+ *  \ingroup hardware_dma
+ *
+ * If enabled, the sniff data result appears bit-reversed when read.
+ * This does not affect the way the checksum is calculated.
+ *
+ * \param reverse Set true to enable output bit reversal
+ */
+inline static void dma_sniffer_set_output_reverse_enabled(bool reverse) {
+    if (reverse)
+        hw_set_bits(&dma_hw->sniff_ctrl, DMA_SNIFF_CTRL_OUT_REV_BITS);
+    else
+        hw_clear_bits(&dma_hw->sniff_ctrl, DMA_SNIFF_CTRL_OUT_REV_BITS);
+}
+
 /*! \brief Disable the DMA sniffer
  *  \ingroup hardware_dma
  *
@@ -769,6 +819,27 @@ inline static void dma_sniffer_disable(void) {
     dma_hw->sniff_ctrl = 0;
 }
 
+/*! \brief Set the sniffer's data accumulator with initial value
+ *  \ingroup hardware_dma
+ *
+ * Generally, CRC algorithms are used with the data accumulator initially
+ * seeded with 0xFFFF or 0xFFFFFFFF (for crc16 and crc32 algorithms)
+ *
+ * \param seed_value value to set data accumulator
+ */
+inline static void dma_sniffer_set_data_accumulator(uint32_t seed_value) {
+    dma_hw->sniff_data = seed_value;
+}
+
+/*! \brief Get the sniffer's data accumulator value
+ *  \ingroup hardware_dma
+ *
+ * Read value calculated by the hardware from sniffing the DMA stream
+ */
+inline static uint32_t dma_sniffer_get_data_accumulator(void) {
+    return dma_hw->sniff_data;
+}
+
 /*! \brief Mark a dma timer as used
  *  \ingroup hardware_dma
  *
@@ -806,11 +877,12 @@ int dma_claim_unused_timer(bool required);
  */
 bool dma_timer_is_claimed(uint timer);
 
-/*! \brief Set the divider for the given DMA timer
+/*! \brief Set the multiplier for the given DMA timer
  *  \ingroup hardware_dma
  *
  * The timer will run at the system_clock_freq * numerator / denominator, so this is the speed
- * that data elements will be transferred at via a DMA channel using this timer as a DREQ
+ * that data elements will be transferred at via a DMA channel using this timer as a DREQ. The
+ * multiplier must be less than or equal to one.
  *
  * \param timer the dma timer
  * \param numerator the fraction's numerator
@@ -818,6 +890,7 @@ bool dma_timer_is_claimed(uint timer);
  */
 static inline void dma_timer_set_fraction(uint timer, uint16_t numerator, uint16_t denominator) {
     check_dma_timer_param(timer);
+    invalid_params_if(HARDWARE_DMA, numerator > denominator);
     dma_hw->timer[timer] = (((uint32_t)numerator) << DMA_TIMER0_X_LSB) | (((uint32_t)denominator) << DMA_TIMER0_Y_LSB);
 }
 
@@ -834,6 +907,36 @@ static inline uint dma_get_timer_dreq(uint timer_num) {
     return DREQ_DMA_TIMER0 + timer_num;
 }
 
+/*! \brief Return DMA_IRQ_<irqn>
+ *  \ingroup hardware_dma
+ *
+ * \param irq_index 0 the DMA irq index
+ * \return The \ref irq_num_to use for DMA
+ */
+static inline int dma_get_irq_num(uint irq_index) {
+    valid_params_if(HARDWARE_DMA, irq_index < NUM_DMA_IRQS);
+    return DMA_IRQ_NUM(irq_index);
+}
+
+/*! \brief Performs DMA channel cleanup after use
+ *  \ingroup hardware_dma
+ *
+ * This can be used to cleanup dma channels when they're no longer needed, such that they are in a clean state for reuse.
+ * IRQ's for the channel are disabled, any in flight-transfer is aborted and any outstanding interrupts are cleared.
+ * The channel is then clear to be reused for other purposes.
+ *
+ * \code
+ * if (dma_channel >= 0) {
+ *     dma_channel_cleanup(dma_channel);
+ *     dma_channel_unclaim(dma_channel);
+ *     dma_channel = -1;
+ * }
+ * \endcode
+ *
+ * \param channel DMA channel
+ */
+void dma_channel_cleanup(uint channel);
+
 #ifndef NDEBUG
 void print_dma_ctrl(dma_channel_hw_t *channel);
 #endif
